/***************************************************************************
 *            init.c
 *
 *  Sun Mar 15 03:39:39 2009
 *  Copyright  2009  Dirk Broßwick
 *  <sharandac@snafu.de>
 ****************************************************************************/

/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor Boston, MA 02110-1301,  USA
 */
 
#include <avr/io.h>
#include <avr/interrupt.h>
#include <stdio.h>
#include <avr/pgmspace.h>
#include <avr/version.h>

#include "config.h"

#include "hardware/network/enc28j60.h"
#include "hardware/uart/uart.h"
#ifdef __AVR_ATmega2561__
	#include "hardware/memory/xram.h"
	#include "hardware/vs10xx/vs10xx.h"
	#include "hardware/led/led_core.h"
#endif

#ifdef __AVR_ATmega644__
	#include "hardware/adc/adc.h"
	#include "hardware/gpio/gpio_out.h"
	#include "hardware/1wire/1wire.h"
#endif

#include "system/clock/clock.h"
#include "system/net/network.h"
#include "system/stdout/stdout.h"
#include "system/config/config.h"

#ifdef __AVR_ATmega2561__
// Timerinterrupt mit abstand 1 sekunde, dabei wird die LED 0 getoggelt
void blinkinglights( void )
{
		LED_toggle(0);
}
#endif

void init( void )
{
 	// Interrupts freigeben
	sei();

	// RS232 starten und printf auf RS232 verbiegen
	UART_init();

	STDOUT_INIT ();
	STDOUT_Set_RS232();

	printf_P( PSTR("%c[2J"),27 ); // Bildschirm löschen in Terminalprogram
	printf_P( PSTR("\r\nMicrowebserver build on AVR-libc version: " __AVR_LIBC_VERSION_STRING__ "/" __AVR_LIBC_DATE_STRING__ " $Id: main.c 132 2009-05-22 3:22:47Z sharan $ $Revision: 134 $\r\n"));

	printf_P( PSTR("UART initialisiert\r\n"));

	printf_P( PSTR("STDOUT initialisiert\r\n"));
	
	// Clock starten
	CLOCK_init();

	Config_Init ();

	#ifdef __AVR_ATmega644__
		#ifdef GPIO
			GPIO_out_init();
			printf_P( PSTR("GPIO initialisiert\r\n"));
		#endif
		#ifdef ADC_
			ADC_init();
			printf_P( PSTR("ADC initialisiert\r\n"));
		#endif
		#ifdef ONEWIRE
			if ( ONEWIRE_reset() )
				printf_P( PSTR("1Wire Error\r\n"));
			else
				printf_P( PSTR("1Wire initialisiert\r\n"));
		#endif
	#endif
	
	#ifdef __AVR_ATmega2561__
		// LED_core starten
		LED_init();

		// Callbackfunktion für die blinkende LED eintragen
		CLOCK_RegisterCallbackFunction ( blinkinglights, SECOUND );

		// VS10xx starten
		VS10xx_INIT();
	#endif
	
	network_init();
	
}
